; =======================================================================================
; Title .........: PowerPoint from Word document
; Description ...: An Autoit script to generate a PowerPoint presentation from MSWord document
; Requirement(s).: compatible Autoit v3.3.14.2
; Author(s) .....: Giovanni Rossati (El Condor)
; Version........: 0.1.0
; Date...........: 18 July 2016
; =======================================================================================
#include <MsgBoxConstants.au3>
#include <StringConstants.au3>
#include "formGen.au3"
#include <Array.au3>
; *****************************************************************************
Global Const $ppLayoutText = 2
Global Const $ppLayoutTitle = 1
Global Const $msoAnchorMiddle = 3 ; vertical align to center
Global Const $ppAlignCenter = 2
Global Const $ppAlignLeft = 1
Global Const $ppSaveAsPDF = 32
Global Const $WdCollapseEnd = 0 ; Collapse the range to the ending point
Global $oHandleComError = ObjEvent("AutoIt.Error", "HandleComError") ; Install a custom error handler
Global $endPresentation = False
Global $fl = ""
Global $normal ; style
Global $oDoc ; Word document
$form = "File,Document,,40,,Documents (*.doc\59*.docx\59*.odt)|All (*.*);" _
		 & "CMB,cmbStyles,Styles,,;" _
		 & "T,Styles,Selected styles,25;" _
		 & "CMT,Background,Bakground style,25,No,0=No|13158600=Silver|13107200=Blue|51200=Green|200=Red;" _
		 & "CMB,Effect,Effect on slide,25,No,0=No|3=Gradient Diagonal Up|4=Gradient Diagonal Down|7=Gradient From Center;" _
		 & "CKB,PDF,Save also PDF,10;" _
		 & "CKB,runPPT,Run at end,10,1;" _
		 & "CKB,SaveClose,Save and close,10,1;" _
		 & "B,styleSet,Add style,10,,addStyle;" _
		 & "C,Va,_;" _
		 & "After,styleSet,cmbStyles;" _
		 & "Tooltip,Background,Choice from list or input a RGB number;" _
		 & "Tooltip,styleSet,Add a select style to Selected styles field;"
$parms = formGen("Word File", $form, -1, cbFunction, 100, 100) ; create a form for call a filename
If $parms.Item("fg_button") = "Cancel" Then Exit
$fl = $parms.Item("Document")
$oDoc = ObjGet($fl) ; open the MSWord document
$aArray = StringRegExp($fl, "(.+)\..+$", 1) ; for strip filetype
Global $commentsDict = ObjCreate("Scripting.Dictionary") ; Create a Scripting Dictionary
For $comment In $oDoc.Comments
	$commentsDict.Item($comment.scope.text) = $comment.range.text
Next

For $bookmark In $oDoc.bookmarks
	$commentsDict.Item($bookmark.name) = $bookmark.range.text
Next
$flPPT = $aArray[0] & ".ppt"
Global $oPPT = ObjCreate("PowerPoint.Application")
$PPT = $oPPT.Presentations.Add(True) ; add a Presentation obj
; Lene Fredborg, DocTools, www.thedoctools.com
$heading1 = $oDoc.Styles(-2).NameLocal
$heading2 = $oDoc.Styles(-3).NameLocal
$heading3 = $oDoc.Styles(-4).NameLocal
;$normal = $oDoc.Styles(-1).NameLocal		; is in cbFunction
$BodyText = $oDoc.Styles(-67).NameLocal
$title = $oDoc.Styles(-63).NameLocal
Global $index = 0 ; progressive slide index
Global $indexHead ; index of slide whith description
Global $subIndex ; index of slide with subindexes
Global $superIndex = 0 ; index of possibly title slide with subindexes
$oParagraphs = $oDoc.Paragraphs
$nParag = $oParagraphs.count
$EventObject = ObjEvent($oPPT, "PWTevent_") ; Start receiving Events.
; set slide background and effect
If $parms.Item("Background") > 0 Then $PPT.SlideMaster.Background.Fill.foreColor.RGB = $parms.Item("Background")
If $parms.Item("Effect") > 0 Then $PPT.SlideMaster.Background.Fill.oneColorGradient($parms.Item("Effect"), 1, 1)
$takeParag = False ; for get first paragraph after heading
$noTxt = False ; if in heading2 there aren't data take heading3
For $oParag In $oParagraphs
	$style = $oParag.Style.NameLocal
	Global $parag = $oParag.Range.Text
	$parag = StringStripWS($parag, $STR_STRIPTRAILING)
	Select
		Case $style = $heading1
			$takeParag = True
			headSlide()
			$indexHead = $index
			$index = $index + 1
			$oSlide = $PPT.Slides.Add($index, $ppLayoutText) ;	slide for sub heading
			$oSlide.Shapes(1).TextFrame.TextRange.Text = $parag
			$oSlide.Shapes(1).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignCenter
			$subIndex = $index
			If $superIndex > 0 Then
				$oSlide = $PPT.Slides.Item($superIndex)
				$oSlide.Shapes.Item(2).TextFrame.TextRange.Text = $oSlide.Shapes.Item(2).TextFrame.TextRange.Text & Chr(13) & $parag
			EndIf
		Case $style = $heading2
			$takeParag = True
			$oSlide = $PPT.Slides.Item($subIndex)
			$oSlide.Shapes.Item(2).TextFrame.TextRange.Text = $oSlide.Shapes.Item(2).TextFrame.TextRange.Text & Chr(13) & $parag
			$index = $index + 1
			$oSlide = $PPT.Slides.Add($index, $ppLayoutTitle)
			$indexHead = $index
			$oSlide.Shapes(1).TextFrame.TextRange.Text = $parag
			$oSlide.Shapes(1).TextFrame2.VerticalAnchor = $msoAnchorMiddle ; vertical align
			$oSlide.Shapes(2).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignLeft
			If $commentsDict.Exists($parag) Then
				$oSlide.Shapes(2).TextFrame.TextRange.Text = $commentsDict.Item($parag)
			EndIf
		Case $style = $heading3 ; when there aren't data
			$oSlide = $PPT.Slides.Item($index)
			$txt = $oSlide.Shapes.Item(2).TextFrame.TextRange.Text
			If $txt = "" Or $noTxt = True Then
				$noTxt = True
				$oSlide.Shapes(2).TextFrame.TextRange.Text = $txt & Chr(13) & $parag
			EndIf
		Case $style = $title
			headSlide()
			$takeParag = True
			$indexHead = $index
			$index = $index + 1
			$oSlide = $PPT.Slides.Add($index, $ppLayoutText) ;	slide for sub heading
			$oSlide.Shapes(1).TextFrame.TextRange.Text = $parag
			$oSlide.Shapes(1).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignCenter
			$superIndex = $index
		Case $style = $normal Or StringInStr($parms.Item("Styles"), $style) > 0
			If $takeParag = True Then
				$noTxt = False
				$takeParag = False
				$oSlide = $PPT.Slides($indexHead)
				$oSlide.Shapes(2).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignLeft
				If $oSlide.Shapes(2).TextFrame.TextRange.Text = "" Then $oSlide.Shapes.Item(2).TextFrame.TextRange.Text = $parag
			EndIf
		Case Else
			$takeParag = False
	EndSelect
Next
$oDoc.Close()
; remove empty slides
For $i = 1 To $index
	$oSlide = $PPT.Slides("Slide" & $i)
	If Not $oSlide.Shapes(2).TextFrame.HasText Then $oSlide.delete
Next
If $parms.Item("runPPT") = 1 Then
	$PPT.SlideShowSettings.Run() ;start the slide show
	While Not $endPresentation	; wait end of presentation
		Sleep(10)
	WEnd
EndIf
;==========Save presentation and exit==================
If $parms.Item("PDF") = 1 Then $PPT.SaveAs($aArray[0] & ".pdf", $ppSaveAsPDF)
If $parms.Item("SaveClose") = 1 Then
	$PPT.SaveAs($flPPT)
	$PPT.Close()
	$oPPT.Quit() ;Exit PowerPoint
EndIf
Func PWTevent_SlideShowEnd($pres) ; event handler
	$endPresentation = True
EndFunc   ;==>PWTevent_SlideShowEnd
Func headSlide()
	$index = $index + 1
	$oSlide = $PPT.Slides.Add($index, $ppLayoutTitle) ;	title slide
	$oSlide.Shapes.Item(1).TextFrame.TextRange.Text = $parag
	$oSlide.Shapes(1).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignCenter
	$oSlide.Shapes(1).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignCenter
	$oSlide.Shapes(1).TextFrame2.VerticalAnchor = $msoAnchorMiddle ; vertical align
	If $commentsDict.Exists($parag) Then
		$oSlide.Shapes(2).TextFrame.TextRange.ParagraphFormat.Alignment = $ppAlignLeft
		$oSlide.Shapes(2).TextFrame.TextRange.Text = $commentsDict.Item($parag)
	EndIf
EndFunc   ;==>headSlide
Func addStyle($aData)
	$cmbStyles = $aData.Item("cmbStyles")
	$styles = $aData.Item("Styles")
	If $cmbStyles <> "" And StringInStr($styles, $cmbStyles) = 0 Then
		If $styles <> "" Then $styles &= ","
		GUICtrlSetData(WidgetID("Styles"), $styles & $cmbStyles)
	EndIf
EndFunc   ;==>addStyle
Func cbFunction($msg)
	If $msg <> 0 Then
		If $msg > 0 Then
			$resp = $msg
			$content = ""
			If widgetName($msg) = "Document" Then
				GUISetCursor(15) ; Wait
				$fl = GetCtrlData($fhaRecords[$fhDictCRef.Item("Document")])
				$oDoc = ObjGet($fl)
				$st = ""
				For $style In $oDoc.styles
					$st &= "|" & $style.nameLocal
				Next
				replaceCombo("cmbStyles", $st)
				$normal = $oDoc.Styles(-1).NameLocal
				GUICtrlSetData(widgetID("Styles"), $normal)
				GUISetCursor() ; reset cursor
			EndIf
		EndIf
	EndIf
EndFunc   ;==>cbFunction
Func HandleComError() ; This is a custom error handler
	$sHexNumber = Hex($oHandleComError.number, 8)
	MsgBox($MB_OK, "", "COM Error at line " & $oHandleComError.scriptline & @CRLF & _
			"Code is: " & $sHexNumber & " " & $oHandleComError & " " & $oHandleComError.windescription)
EndFunc   ;==>HandleComError